@@ -1,17 +1,17 @@
 /*
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; You may only use version 2 of the License,
-	you have no option to use any other version.
+        This program is free software; you can redistribute it and/or modify
+        it under the terms of the GNU General Public License as published by
+        the Free Software Foundation; You may only use version 2 of the License,
+        you have no option to use any other version.
 
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
+        This program is distributed in the hope that it will be useful,
+        but WITHOUT ANY WARRANTY; without even the implied warranty of
+        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+        GNU General Public License for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software
-	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        You should have received a copy of the GNU General Public License
+        along with this program; if not, write to the Free Software
+        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
         xfce4 mcs plugin   - (c) 2002 Olivier Fourdan
 
@@ -43,20 +43,20 @@
 #define RCFILE2 "mouse.xml"
 #define PLUGIN_NAME "mouse"
 
-#define DEFAULT_ACCELERATION	9
-#define DEFAULT_THRESHOLD	6
-#define DEFAULT_DBL_CLICKTIME 	300
-#define DEFAULT_DND_THRESHOLD 	8
+#define DEFAULT_ACCELERATION    2
+#define DEFAULT_THRESHOLD       4
+#define DEFAULT_DENOMINATOR     1
+#define DEFAULT_DBL_CLICKTIME   300
+#define DEFAULT_DND_THRESHOLD   8
 
-#define ACCEL_MIN	1
-#define ACCEL_MAX	30
-#define DENOMINATOR	3
-#define THRESH_MIN	1
-#define THRESH_MAX	20
-#define DND_THRESH_MIN	1
-#define DND_THRESH_MAX	20
-#define DBL_CLICK_MIN	100
-#define DBL_CLICK_MAX	2000
+#define ACCEL_MIN       1
+#define ACCEL_MAX       30
+#define THRESH_MIN      1
+#define THRESH_MAX      20
+#define DND_THRESH_MIN  1
+#define DND_THRESH_MAX  20
+#define DBL_CLICK_MIN   100
+#define DBL_CLICK_MAX   2000
 #define DEFAULT_ICON_SIZE 48
 
 static void create_channel(McsPlugin * mcs_plugin);
@@ -69,6 +69,7 @@ static int dbl_clicktime = DEFAULT_DBL_CLICKTIME;
 static int dnd_threshold = DEFAULT_DND_THRESHOLD;
 static int acceleration = DEFAULT_ACCELERATION;
 static int threshold = DEFAULT_THRESHOLD;
+static int denominator = DEFAULT_DENOMINATOR;
 
 typedef struct _Itf Itf;
 struct _Itf
@@ -115,6 +116,26 @@ struct _Itf
     GtkWidget *closebutton1;
 };
 
+static void get_mouse_values(int *accel_return, int *denom_return, int *thresh_return)
+{
+    gdk_flush();
+    gdk_error_trap_push();
+
+    /* Reset */
+    XChangePointerControl(GDK_DISPLAY(), True, True, -1, -1, -1);
+    gdk_flush();
+    XGetPointerControl(GDK_DISPLAY(), accel_return, denom_return, thresh_return);
+    
+    /* Paranoid check */
+    if (*denom_return < 1)
+    {
+        *denom_return = 1;
+    }
+    
+    gdk_flush();
+    gdk_error_trap_pop();
+}
+
 static void set_mouse_values(gboolean right_handed, int accel, int threshold)
 {
     unsigned char map[5] = { '\0','\0','\0','\0','\0' };
@@ -123,7 +144,7 @@ static void set_mouse_values(gboolean right_handed, int accel, int threshold)
     gdk_flush();
     gdk_error_trap_push();
 
-    XChangePointerControl(GDK_DISPLAY(), 1, 1, accel, DENOMINATOR, threshold);
+    XChangePointerControl(GDK_DISPLAY(), True, True, accel, denominator, threshold);
     buttons = XGetPointerMapping(GDK_DISPLAY(), map, 5);
 
     if(right_handed)
@@ -267,7 +288,7 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
 
     dialog->vbox1 = gtk_vbox_new(FALSE, 0);
     gtk_widget_show(dialog->vbox1);
-    gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->vbox1, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(dialog->dialog_vbox1), dialog->vbox1, TRUE, TRUE, 4);
 
     dialog->frame1 = xfce_framebox_new (_("Button settings"), TRUE);
     gtk_widget_show(dialog->frame1);
@@ -301,51 +322,51 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
 
     dialog->label3 = gtk_label_new(_("Acceleration :"));
     gtk_widget_show(dialog->label3);
-    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label3, 0, 3, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label3, 0, 3, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label3), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label3), 0, 0.5);
 
     dialog->label4 = small_label(_("Slow"));
     gtk_widget_show(dialog->label4);
-    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label4, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label4, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label4), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label4), 1, 0.5);
 
     dialog->hscale1 = gtk_hscale_new(GTK_ADJUSTMENT(gtk_adjustment_new(acceleration, ACCEL_MIN, ACCEL_MAX, 1, 5, 0)));
     gtk_widget_show(dialog->hscale1);
-    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->hscale1, 1, 2, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->hscale1, 1, 2, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 2);
     gtk_scale_set_draw_value(GTK_SCALE(dialog->hscale1), FALSE);
     gtk_scale_set_digits(GTK_SCALE(dialog->hscale1), 0);
     gtk_range_set_update_policy(GTK_RANGE(dialog->hscale1), GTK_UPDATE_DISCONTINUOUS);
 
     dialog->label5 = small_label(_("Fast"));
     gtk_widget_show(dialog->label5);
-    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label5, 2, 3, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label5, 2, 3, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label5), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label5), 0, 0.5);
 
     dialog->label6 = gtk_label_new(_("Threshold :"));
     gtk_widget_show(dialog->label6);
-    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label6, 0, 3, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label6, 0, 3, 2, 3, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label6), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label6), 0, 0.5);
 
     dialog->label7 = small_label(_("Low"));
     gtk_widget_show(dialog->label7);
-    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label7, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label7, 0, 1, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label7), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label7), 1, 0.5);
 
     dialog->hscale2 = gtk_hscale_new(GTK_ADJUSTMENT(gtk_adjustment_new(threshold, THRESH_MIN, THRESH_MAX, 1, 5, 0)));
     gtk_widget_show(dialog->hscale2);
-    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->hscale2, 1, 2, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->hscale2, 1, 2, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 2);
     gtk_scale_set_draw_value(GTK_SCALE(dialog->hscale2), FALSE);
     gtk_scale_set_digits(GTK_SCALE(dialog->hscale2), 0);
     gtk_range_set_update_policy(GTK_RANGE(dialog->hscale2), GTK_UPDATE_DISCONTINUOUS);
 
     dialog->label8 = small_label(_("High"));
     gtk_widget_show(dialog->label8);
-    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label8, 2, 3, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table1), dialog->label8, 2, 3, 3, 4, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label8), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label8), 0, 0.5);
 
@@ -359,26 +380,26 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
 
     dialog->label11 = small_label(_("Low"));
     gtk_widget_show(dialog->label11);
-    gtk_table_attach(GTK_TABLE(dialog->table2), dialog->label11, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table2), dialog->label11, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label11), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label11), 1, 0.5);
 
     dialog->hscale3 = gtk_hscale_new(GTK_ADJUSTMENT(gtk_adjustment_new(dnd_threshold, DND_THRESH_MIN, DND_THRESH_MAX, 1, 5, 0)));
     gtk_widget_show(dialog->hscale3);
-    gtk_table_attach(GTK_TABLE(dialog->table2), dialog->hscale3, 1, 2, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table2), dialog->hscale3, 1, 2, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 2);
     gtk_scale_set_draw_value(GTK_SCALE(dialog->hscale3), FALSE);
     gtk_scale_set_digits(GTK_SCALE(dialog->hscale3), 0);
     gtk_range_set_update_policy(GTK_RANGE(dialog->hscale3), GTK_UPDATE_DISCONTINUOUS);
 
     dialog->label12 = small_label(_("High"));
     gtk_widget_show(dialog->label12);
-    gtk_table_attach(GTK_TABLE(dialog->table2), dialog->label12, 2, 3, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table2), dialog->label12, 2, 3, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label12), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label12), 0, 0.5);
 
     dialog->label10 = gtk_label_new(_("Threshold :"));
     gtk_widget_show(dialog->label10);
-    gtk_table_attach(GTK_TABLE(dialog->table2), dialog->label10, 0, 3, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table2), dialog->label10, 0, 3, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label10), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label10), 0, 0.5);
 
@@ -392,25 +413,25 @@ Itf *create_mouse_dialog(McsPlugin * mcs_plugin)
 
     dialog->label14 = gtk_label_new(_("Speed :"));
     gtk_widget_show(dialog->label14);
-    gtk_table_attach(GTK_TABLE(dialog->table3), dialog->label14, 0, 3, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table3), dialog->label14, 0, 3, 0, 1, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label14), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label14), 0, 0.5);
 
     dialog->label15 = small_label(_("Slow"));
     gtk_widget_show(dialog->label15);
-    gtk_table_attach(GTK_TABLE(dialog->table3), dialog->label15, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table3), dialog->label15, 0, 1, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label15), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label15), 1, 0.5);
 
     dialog->label16 = small_label(_("Fast"));
     gtk_widget_show(dialog->label16);
-    gtk_table_attach(GTK_TABLE(dialog->table3), dialog->label16, 2, 3, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table3), dialog->label16, 2, 3, 1, 2, (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 0, 2);
     gtk_label_set_justify(GTK_LABEL(dialog->label16), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment(GTK_MISC(dialog->label16), 0, 0.5);
 
     dialog->hscale4 = gtk_hscale_new(GTK_ADJUSTMENT(gtk_adjustment_new(dbl_clicktime, DBL_CLICK_MIN, DBL_CLICK_MAX, 10, 100, 0)));
     gtk_widget_show(dialog->hscale4);
-    gtk_table_attach(GTK_TABLE(dialog->table3), dialog->hscale4, 1, 2, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 0);
+    gtk_table_attach(GTK_TABLE(dialog->table3), dialog->hscale4, 1, 2, 1, 2, (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (GTK_FILL), 0, 2);
     gtk_scale_set_draw_value(GTK_SCALE(dialog->hscale4), FALSE);
     gtk_scale_set_digits(GTK_SCALE(dialog->hscale4), 0);
     gtk_range_set_update_policy(GTK_RANGE(dialog->hscale4), GTK_UPDATE_DISCONTINUOUS);
@@ -484,8 +505,11 @@ create_channel(McsPlugin *mcs_plugin)
     mcs_manager_add_channel_from_file(mcs_plugin->manager, CHANNEL2, rcfile);
     g_free(rcfile);
 
+    /* Retrieve server defaults */
+    get_mouse_values(&acceleration, &denominator, &threshold);
+    
     setting = mcs_manager_setting_lookup(mcs_plugin->manager,
-		    "Net/DoubleClickTime", CHANNEL1);
+                    "Net/DoubleClickTime", CHANNEL1);
     if (setting)
         dbl_clicktime = setting->data.v_int;
     else
@@ -516,25 +540,35 @@ create_channel(McsPlugin *mcs_plugin)
         mcs_manager_set_int(mcs_plugin->manager, "Mouse/RightHanded", CHANNEL2, right_handed ? 1 : 0);
     }
 
-    setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Mouse/Acceleration", CHANNEL2);
-    if(setting)
+    setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Mouse/Denominator", CHANNEL2);
+    if((setting) && (setting->data.v_int))
     {
-        acceleration = setting->data.v_int;
-    }
-    else
-    {
-        acceleration = DEFAULT_ACCELERATION;
-        mcs_manager_set_int(mcs_plugin->manager, "Mouse/Acceleration", CHANNEL2, acceleration);
-    }
+        denominator = setting->data.v_int;
+        setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Mouse/Acceleration", CHANNEL2);
+        if(setting)
+        {
+            acceleration = setting->data.v_int;
+        }
+        else
+        {
+            mcs_manager_set_int(mcs_plugin->manager, "Mouse/Acceleration", CHANNEL2, acceleration);
+        }
 
-    setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Mouse/Threshold", CHANNEL2);
-    if(setting)
-    {
-        threshold = setting->data.v_int;
+        setting = mcs_manager_setting_lookup(mcs_plugin->manager, "Mouse/Threshold", CHANNEL2);
+        if(setting)
+        {
+            threshold = setting->data.v_int;
+        }
+        else
+        {
+            mcs_manager_set_int(mcs_plugin->manager, "Mouse/Threshold", CHANNEL2, threshold);
+        }
     }
     else
     {
-        threshold = DEFAULT_THRESHOLD;
+        /* Set default server values */
+        mcs_manager_set_int(mcs_plugin->manager, "Mouse/Denominator", CHANNEL2, denominator);
+        mcs_manager_set_int(mcs_plugin->manager, "Mouse/Acceleration", CHANNEL2, acceleration);
         mcs_manager_set_int(mcs_plugin->manager, "Mouse/Threshold", CHANNEL2, threshold);
     }
 
@@ -549,12 +583,12 @@ write_options(McsPlugin *mcs_plugin)
 
     rcfile = xfce_get_userfile(RCDIR, RCFILE1, NULL);
     result = mcs_manager_save_channel_to_file(mcs_plugin->manager, CHANNEL1,
-		    rcfile);
+                    rcfile);
     g_free(rcfile);
 
     rcfile = xfce_get_userfile(RCDIR, RCFILE2, NULL);
     result &= mcs_manager_save_channel_to_file(mcs_plugin->manager, CHANNEL2,
-		    rcfile);
+                    rcfile);
     g_free(rcfile);
 
     return(result);
@@ -566,10 +600,10 @@ run_dialog(McsPlugin *mcs_plugin)
     Itf *dialog;
 
     if (!is_running) {
-	is_running = TRUE;
+        is_running = TRUE;
 
-    	dialog = create_mouse_dialog(mcs_plugin);
-    	setup_dialog(dialog);
+        dialog = create_mouse_dialog(mcs_plugin);
+        setup_dialog(dialog);
     }
 }
 
